/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import QtTest 1.0

WebbrowserAppTestCase {
    property var window: null
    property var contextMenu: null

    // In narrow layout, the tabslist is a stack: the current tab is always at
    // the top. To ensure that a newly-open tab has been moved to the top,
    // wait for the count of this spy to be increased by 2.
    SignalSpy {
        id: currentIndexSpy
        target: window ? window.tabsModel : null
        signalName: "currentIndexChanged"
    }

    function init() {
        base.init();
        app.init([baseUrl + "test1.html"], true, false);
        tryCompare(app.allWindows, "length", 1);
        window = app.allWindows[0];
        tryCompare(window, "visible", true);
        waitFor(function() { return (window.currentWebview != null); });
        tryCompare(window.currentWebview, "loading", false);
    }

    function cleanup() {
        window.close();
        tryCompare(this, "window", null);
        base.cleanup();
    }

    function clear_address_bar() {
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        verify(addressbar.activeFocus);
        var clearButton = findChild(addressbar, "clear_button");
        clickItem(clearButton);
        tryCompare(addressbar, "text", "");
    }

    function focus_address_bar(clear) {
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        clickItem(addressbar);
        tryCompare(addressbar, "activeFocus", true);
        if (clear) {
            clear_address_bar();
        }
        return addressbar;
    }

    function browse_to(path, waitLoaded) {
        var url = path;
        if (path.indexOf("http://") != 0) {
            url = baseUrl + path;
        }
        focus_address_bar();
        typeString(url);
        keyClick(Qt.Key_Enter);
        tryCompare(window.currentWebview, "url", url);
        if (waitLoaded) {
            tryCompare(window.currentWebview, "loading", false);
        }
    }

    function check_current_url(url) {
        verify(waitFor(function() { return (window.currentWebview != null); }));
        verify(waitFor(function() { return (window.currentWebview.url == url); }));
    }

    function open_drawer(action) {
        var chrome = findChild(window.contentItem, "chrome");
        var drawerButton = findChild(chrome, "drawerButton");
        clickItem(drawerButton);
        var drawer = findChild(chrome, "drawer");
        tryCompare(drawer, "clip", false);
        if (action) {
            clickItem(findChild(drawer, action));
        } else {
            return drawer;
        }
    }

    function openContextMenu() {
        clickItem(window.currentWebview, Qt.RightButton);
        contextMenu = waitForChild(window.contentItem, "contextMenu");
        verify(contextMenu != null);
        waitForRendering(contextMenu);
    }

    function open_tabs_view() {
        verify(!window.wide);
        // XXX: assume there is a mouse connected
        // FIXME: handle the touch case with a bottom-edge drag upwards
        var bottomEdgeBar = findChild(window.contentItem, "bottomEdgeBar");
        verify(bottomEdgeBar.visible);
        verify(bottomEdgeBar.enabled);
        clickItem(bottomEdgeBar);
        var tabsView = waitForChild(window.contentItem, "tabsList");
        tryCompare(tabsView, "visible", true);
        // Give some time for the view to settle so that all previews reached
        // their initial position (the animation has a duration of
        // UbuntuAnimation.BriskDuration, i.e. 333ms).
        wait(500);
        return tabsView;
    }

    function open_new_tab() {
        currentIndexSpy.clear();
        var count = window.tabsModel.count;
        var newTabButton = null;
        if (window.wide) {
            var chrome = findChild(window.contentItem, "chrome");
            newTabButton = findChild(chrome, "newTabButton");
            clickItem(newTabButton);
        } else {
            var tabsView = open_tabs_view();
            newTabButton = findChild(tabsView.parent, "newTabButton");
            clickItem(newTabButton);
            tryCompare(tabsView, "visible", false);
        }
        tryCompare(window.tabsModel, "count", count + 1);
        if (!window.wide) {
            tryCompare(currentIndexSpy, "count", 2);
        }
        tryCompare(window.tabsModel, "currentIndex", window.wide ? count : 0);
        waitFor(function() { return (window.currentWebview != null); });
        verify(waitForChild(window.contentItem, "newTabView") != null);
    }

    function bookmark_current_page() {
        var chrome = findChild(window.contentItem, "chrome");
        verify(!chrome.bookmarked);
        var addressbar = findChild(chrome, "addressbar");
        var bookmarkToggle = findChild(addressbar, "bookmarkToggle");
        tryCompare(bookmarkToggle, "visible", true);
        waitForRendering(bookmarkToggle);
        clickItem(bookmarkToggle);
        var bookmarkOptions = waitForChild(window.contentItem, "bookmarkOptions");
        verify(bookmarkOptions != null);
        tryCompare(bookmarkOptions, "visible", true);
        waitForRendering(bookmarkOptions);
        return bookmarkOptions;
    }
}
