/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "BookmarkOptions"

    bookmarksDbFile: Qt.resolvedUrl("bookmarkoptions-bookmarks.sqlite")

    property var bookmarkOptions: null
    property var newFolderDialog: null

    function open_new_tab_and_expand_bookmarks_view() {
        open_new_tab();
        var newTabView = findChild(window.contentItem, "newTabView");
        if (!window.wide) {
            var moreButton = findChild(newTabView, "bookmarks.moreButton");
            tryCompare(moreButton, "visible", true);
            clickItem(moreButton);
            waitForRendering(newTabView);
        }
        return newTabView;
    }

    function get_folder_list(newTabView) {
        var folderList = null;
        if (window.wide) {
            var sections = findChild(newTabView, "sections");
            tryCompare(sections, "visible", true);
            if (sections.selectedIndex != 1) {
                // Switch to the "bookmarks" section of the new tab view
                var sectionButton = waitForChild(newTabView, "section_button_1");
                verify(sectionButton != null);
                tryCompare(sectionButton, "visible", true);
                clickItem(sectionButton);
                tryCompare(sections, "selectedIndex", 1);
            }
            folderList = findChild(newTabView, "foldersList");
        } else {
            folderList = findChild(newTabView, "bookmarksFolderListView");
        }
        verify(folderList != null);
        tryCompare(folderList, "visible", true);
        return folderList;
    }

    function assert_folder_count(newTabView, count) {
        var folderList = get_folder_list(newTabView);
        tryCompare(folderList, "count", count);
    }

    function get_bookmark_folder(newTabView, folderName) {
        var folderList = get_folder_list(newTabView);
        waitForRendering(folderList);
        // Find and select the designated folder
        var folders, folder = null, i, f;
        if (window.wide) {
            folders = folderList.contentItem.children;
            folder = null;
            for (i = 0; i < folders.length; ++i) {
                f = folders[i];
                if ((f.objectName.indexOf("folderItem") == 0) &&
                    (f.name == folderName)) {
                    folder = f;
                    break;
                }
            }
        } else {
            folders = folderList.contentItem.children;
            folder = null;
            for (i = 0; i < folders.length; ++i) {
                f = folders[i];
                if ((f.objectName == "bookmarkFolderDelegateLoader") &&
                    (f.item.folderName == folderName)) {
                    folder = f;
                    break;
                }
            }
        }
        verify(folder != null);
        return folder;
    }

    function assert_bookmark_count_in_folder(newTabView, folderName, count) {
        var folder = get_bookmark_folder(newTabView, folderName);
        if (window.wide) {
            clickItem(folder);
            var bookmarkList = findChild(newTabView, "bookmarksList");
            tryCompare(bookmarkList, "count", count);
        } else {
            // This assumes the folder is already expanded
            var bookmarkListView = findChild(folder, "bookmarksInFolder");
            tryCompare(bookmarkListView, "count", count);
        }
    }

    function toggle_bookmark_folder(newTabView, folderName) {
        if (!window.wide) {
            var folder = get_bookmark_folder(newTabView, folderName);
            var header = findChild(folder, "bookmarkFolderHeader");
            verify(header != null);
            clickItem(header);
        }
    }

    function test_save_bookmarked_url_in_default_folder() {
        var newTabView = open_new_tab_and_expand_bookmarks_view();
        assert_bookmark_count_in_folder(newTabView, "", 5);
        browse_to("test2.html", true);
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "bookmarked", false);

        bookmarkOptions = bookmark_current_page();
        var dismissButton = findChild(bookmarkOptions, "bookmarkOptions.okButton");
        clickItem(dismissButton);
        tryCompare(this, "bookmarkOptions", null);
        tryCompare(chrome, "bookmarked", true);

        newTabView = open_new_tab_and_expand_bookmarks_view();
        assert_bookmark_count_in_folder(newTabView, "", 6);
    }

    function test_save_bookmarked_url_in_existing_folder() {
        var newTabView = open_new_tab_and_expand_bookmarks_view();
        assert_folder_count(newTabView, 3);
        toggle_bookmark_folder(newTabView, "Actinide");
        assert_bookmark_count_in_folder(newTabView, "Actinide", 1);
        browse_to("page2.html", true);
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "bookmarked", false);
        bookmarkOptions = bookmark_current_page();

        var optionSelector = findChild(bookmarkOptions, "folderOptionSelector");
        verify(optionSelector != null);
        tryCompare(optionSelector, "visible", true);
        verify(!optionSelector.currentlyExpanded);
        clickItem(optionSelector);
        tryCompare(optionSelector, "currentlyExpanded", true);
        tryCompare(optionSelector, "height", optionSelector.itemHeight * 3);
        var folderDelegate = findChild(optionSelector, "optionSelectorDelegate_Actinide");
        clickItem(folderDelegate);
        tryCompare(optionSelector, "currentlyExpanded", false);
        tryCompare(optionSelector, "height", optionSelector.itemHeight);

        var dismissButton = findChild(bookmarkOptions, "bookmarkOptions.okButton");
        clickItem(dismissButton);
        tryCompare(this, "bookmarkOptions", null);
        tryCompare(chrome, "bookmarked", true);

        newTabView = open_new_tab_and_expand_bookmarks_view();
        assert_folder_count(newTabView, 3);
        toggle_bookmark_folder(newTabView, "Actinide");
        assert_bookmark_count_in_folder(newTabView, "Actinide", 2);
    }

    function test_save_bookmarked_url_in_new_folder() {
        var newTabView = open_new_tab_and_expand_bookmarks_view();
        assert_folder_count(newTabView, 3);
        browse_to("test2.html", true);
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "bookmarked", false);
        bookmarkOptions = bookmark_current_page();

        // First test cancelling the creation of a new folder
        var newFolderButton = findChild(bookmarkOptions, "bookmarkOptions.newButton");
        clickItem(newFolderButton);
        newFolderDialog = waitForChild(window.contentItem, "newFolderDialog");
        var cancelButton = findChild(newFolderDialog, "newFolderDialog.cancelButton");
        clickItem(cancelButton);
        tryCompare(this, "newFolderDialog", null);

        // Then test actually creating a new folder
        clickItem(newFolderButton);
        newFolderDialog = waitForChild(window.contentItem, "newFolderDialog");
        var textField = findChild(newFolderDialog, "newFolderDialog.text");
        tryCompare(textField, "activeFocus", true);
        tryCompare(textField, "text", "");
        typeString("NewFolder");
        var saveButton = findChild(newFolderDialog, "newFolderDialog.saveButton");
        clickItem(saveButton);
        tryCompare(this, "newFolderDialog", null);
        var dismissButton = findChild(bookmarkOptions, "bookmarkOptions.okButton");
        clickItem(dismissButton);
        tryCompare(this, "bookmarkOptions", null);
        tryCompare(chrome, "bookmarked", true);

        newTabView = open_new_tab_and_expand_bookmarks_view();
        assert_folder_count(newTabView, 4);
        toggle_bookmark_folder(newTabView, "NewFolder");
        assert_bookmark_count_in_folder(newTabView, "NewFolder", 1);
    }

    function test_set_bookmark_title() {
        browse_to("blanktargetlink.html", true);
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "bookmarked", false);
        bookmarkOptions = bookmark_current_page();
        var titleField = findChild(bookmarkOptions, "titleTextField");
        tryCompare(titleField, "text", baseUrl.substr("http://".length) + "blanktargetlink.html");
        clickItem(titleField);
        tryCompare(titleField, "activeFocus", true);
        var clearButton = findChild(titleField, "clear_button");
        clickItem(clearButton);
        typeString("NewTitle");
        var dismissButton = findChild(bookmarkOptions, "bookmarkOptions.okButton");
        clickItem(dismissButton);
        tryCompare(this, "bookmarkOptions", null);
        tryCompare(chrome, "bookmarked", true);

        var newTabView = open_new_tab_and_expand_bookmarks_view();
        assert_bookmark_count_in_folder(newTabView, "", 6);
        var folder = get_bookmark_folder(newTabView, "");
        var bookmarkList = null;
        var bookmark = null;
        if (window.wide) {
            bookmarkList = findChild(newTabView, "bookmarksList");
            bookmark = findChild(bookmarkList, "bookmarkItem_1");
        } else {
            bookmarkList = findChild(folder, "bookmarksInFolder");
            bookmark = findChild(bookmarkList, "urlDelegate_1");
        }
        verify(bookmark != null);
        tryCompare(bookmark, "title", "NewTitle");
    }

    function test_bookmark_options_from_contextual_menu() {
        browse_to("blanktargetlink.html", true);
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "bookmarked", false);

        // Invoke the context menu over the link, which covers the entire page
        openContextMenu();
        var actionItem = findChild(contextMenu, "BookmarkLinkContextualAction_item");
        verify(actionItem && actionItem.visible);
        clickItem(actionItem);
        tryCompare(this, "contextMenu", null);
        bookmarkOptions = waitForChild(window.contentItem, "bookmarkOptions");
        verify(bookmarkOptions != null);
        tryCompare(bookmarkOptions, "visible", true);
        waitForRendering(bookmarkOptions);
        var dismissButton = findChild(bookmarkOptions, "bookmarkOptions.okButton");
        clickItem(dismissButton);
        tryCompare(this, "bookmarkOptions", null);

        // Re-open the context menu and verify that the bookmark options is
        // disabled as we have already bookmarked this link.
        openContextMenu();
        actionItem = findChild(contextMenu, "BookmarkLinkContextualAction_item");
        verify(actionItem && !actionItem.visible);

        // Dismiss the context menu
        if (window.wide) {
            // Dismiss by clicking outside of the menu
            var webview = window.currentWebview
            mouseClick(webview, contextMenu.x / 2, webview.y + webview.height / 2);
        } else {
            var cancelAction = findChild(contextMenu, "cancelAction");
            verify(cancelAction && cancelAction.visible);
            clickItem(cancelAction);
        }
        tryCompare(this, "contextMenu", null);

        // Click on the link and verify that the bookmark star is lit on the
        // target page.
        clickItem(window.currentWebview);
        check_current_url(baseUrl + "test2.html");
        tryCompare(chrome, "bookmarked", true);
    }
}
