/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import webbrowserapp.private 0.1

SingleWindowTestCase {
    id: testCase
    name: "ContextMenu"

    property var window2: null
    property var bookmarkOptions: null

    function getContextAction(action) {
        verify(contextMenu != null);
        return findChild(contextMenu, action + "ContextualAction_item");
    }

    function browse_to_and_activate_context_menu(path, title, action) {
        // First, open a second tab that will be used to verify tab ordering
        open_new_tab();
        window.tabsModel.currentIndex = (window.wide ? 0 : 1);

        browse_to(path, true);
        openContextMenu();
        var titleLabel = findChild(contextMenu, "titleLabel");
        compare(titleLabel.text, title);
        var actionItem = getContextAction(action);
        verify(actionItem && actionItem.visible);
        clickItem(actionItem);
        tryCompare(testCase, "contextMenu", null);
    }

    function verify_new_foreground_tab_opened(path) {
        tryCompare(window.tabsModel, "count", 3);
        tryCompare(window.tabsModel, "currentIndex", window.wide ? 1 : 0);
        check_current_url(baseUrl + path);
    }

    function verify_new_background_tab_opened(path) {
        tryCompare(window.tabsModel, "count", 3);
        tryCompare(window.tabsModel, "currentIndex", 0);
        compare(window.tabsModel.get(1).url, baseUrl + path);
    }

    function compare_clipboard_text(expected) {
        verify(waitFor(function() { return (Clipboard.data.text == expected); }));
    }

    function link_tests_test_data() {
        return [
            {path: "link.html", title: baseUrl + "test1.html"},
            {path: "imagelink.html", title: baseUrl + "image.png"},
        ];
    }

    function image_tests_test_data() {
        return [
            {path: "image.html"},
            {path: "imagelink.html"},
        ];
    }

    function test_open_link_in_new_tab_data() {
        return link_tests_test_data();
    }

    function test_open_link_in_new_tab(data) {
        browse_to_and_activate_context_menu(data.path, data.title, "OpenLinkInNewTab");
        verify_new_foreground_tab_opened("test1.html");
    }

    function test_open_link_in_new_background_tab_data() {
        return link_tests_test_data();
    }

    function test_open_link_in_new_background_tab(data) {
        browse_to_and_activate_context_menu(data.path, data.title, "OpenLinkInNewBackgroundTab");
        verify_new_background_tab_opened("test1.html");
    }

    function test_open_link_in_another_window_data() {
        return [
            {path: "link.html", title: baseUrl + "test1.html",
             action: "OpenLinkInNewWindow", incognito: false},
            {path: "link.html", title: baseUrl + "test1.html",
             action: "OpenLinkInPrivateWindow", incognito: true},
            {path: "imagelink.html", title: baseUrl + "image.png",
             action: "OpenLinkInNewWindow", incognito: false},
            {path: "imagelink.html", title: baseUrl + "image.png",
             action: "OpenLinkInPrivateWindow", incognito: true},
        ];
    }

    function test_open_link_in_another_window(data) {
        browse_to_and_activate_context_menu(data.path, data.title, data.action);
        tryCompare(app.allWindows, "length", 2);
        window2 = app.allWindows[1];
        compare(window2.incognito, data.incognito);
        tryCompare(window2, "visible", true);
        waitFor(function() { return (window2.currentWebview != null); });
        tryCompare(window2.currentWebview, "url", baseUrl + "test1.html");
        window2.close();
        tryCompare(this, "window2", null);
    }

    function test_bookmark_link_data() {
        return link_tests_test_data();
    }

    function test_bookmark_link(data) {
        browse_to_and_activate_context_menu(data.path, data.title, "BookmarkLink");
        bookmarkOptions = waitForChild(window.contentItem, "bookmarkOptions");
        var dismissButton = findChild(bookmarkOptions, "bookmarkOptions.okButton");
        clickItem(dismissButton);
        tryCompare(this, "bookmarkOptions", null);
        var chrome = findChild(window.contentItem, "chrome");
        verify(!chrome.bookmarked);
        browse_to("test1.html", true);
        tryCompare(chrome, "bookmarked", true);
        BookmarksModel.remove(baseUrl + "test1.html");
    }

    function test_copy_link_data() {
        return link_tests_test_data();
    }

    function test_copy_link(data) {
        Clipboard.clear();
        browse_to_and_activate_context_menu(data.path, data.title, "CopyLink");
        compare_clipboard_text(baseUrl + "test1.html");
    }

    function test_share_link_data() {
        return link_tests_test_data();
    }

    function test_share_link(data) {
        browse_to_and_activate_context_menu(data.path, data.title, "Share");
        verify(waitForChild(window.contentItem, "contentShareDialog"));
    }

    function test_open_image_in_new_tab_data() {
        return image_tests_test_data();
    }

    function test_open_image_in_new_tab(data) {
        browse_to_and_activate_context_menu(data.path, baseUrl + "image.png", "OpenImageInNewTab");
        verify_new_foreground_tab_opened("image.png");
    }

    function test_copy_image_data() {
        return image_tests_test_data();
    }

    function test_copy_image(data) {
        Clipboard.clear();
        compare(Clipboard.data.formats.length, 0);
        browse_to_and_activate_context_menu(data.path, baseUrl + "image.png", "CopyImage");
        verify(Clipboard.data.formats.length > 0);
        compare(Clipboard.data.formats[0], "application/x-qt-image");
    }

    function test_textarea_actions() {
        browse_to("textarea.html", true);
        var actions = ["SelectAll", "Cut", "Undo", "Redo",
                       "Paste", "SelectAll", "Copy", "Erase"]
        for (var i in actions) {
            openContextMenu();
            var titleLabel = findChild(contextMenu, "titleLabel");
            verify(!titleLabel.visible);
            var actionItem = getContextAction(actions[i]);
            verify(actionItem && actionItem.visible);
            clickItem(actionItem);
            tryCompare(testCase, "contextMenu", null);
            tryCompare(window.currentWebview, "activeFocus", true);
        }
    }

    function select_text_and_activate_context_menu(action) {
        browse_to("test1.html", true);
        var webview = window.currentWebview;
        var chrome = findChild(window.contentItem, "chrome");
        var x = 5;
        var y = chrome.height + 5;
        var dx = webview.width - 10;
        var dy = webview.height - chrome.height - 10;
        mouseDrag(webview, x, y, dx, dy, Qt.LeftButton, Qt.NoModifier, 10);
        openContextMenu();
        var actionItem = getContextAction(action);
        verify(actionItem && actionItem.visible);
        clickItem(actionItem);
        tryCompare(testCase, "contextMenu", null);
    }

    function test_textselection_copy() {
        Clipboard.clear();
        select_text_and_activate_context_menu("Copy");
        compare_clipboard_text("test page 1");
    }

    function test_textselection_share() {
        select_text_and_activate_context_menu("Share");
        verify(waitForChild(window.contentItem, "contentShareDialog"));
    }
}
