/*
 * Copyright 2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import QtTest 1.0
import webbrowser.qmltests 0.1

SingleWindowTestCase {
    name: "Downloads"

    property var downloadsPage: null
    property var downloadDialog: null
    property var optionsDialog: null

    function open_downloads() {
        open_drawer("downloads");
        downloadsPage = waitForChild(window.contentItem, "downloadsPage");
        verify(downloadsPage != null);
        waitForRendering(downloadsPage);
    }

    function test_open_close_downloads_page() {
        open_downloads();
        var pageHeader = findChild(downloadsPage, "pageHeader");
        var backButton = waitForChild(pageHeader, "back_button");
        tryCompare(backButton, "visible", true);
        clickItem(backButton);
        tryCompare(this, "downloadsPage", null);
    }

    function test_mimetype_download_data() {
        return [
            { path: "test.pdf" },
            { path: "downloadpdfgenericmime.py" },
        ];
    }

    function test_mimetype_download(data) {
        browse_to(data.path, false);
        downloadDialog = waitForChild(window.contentItem, "downloadDialog");
        compare(downloadDialog.mimeType, "application/pdf");
        optionsDialog =
            waitForChild(window.contentItem, "downloadOptionsDialog");
        tryCompare(optionsDialog, "visible", true);
    }

    function test_filename() {
        browse_to("test.pdf", false);
        downloadDialog = waitForChild(window.contentItem, "downloadDialog");
        compare(downloadDialog.filename, "test.pdf");
    }

    function test_close_dialog() {
        browse_to("test.pdf", false);
        optionsDialog =
            waitForChild(window.contentItem, "downloadOptionsDialog");
        tryCompare(optionsDialog, "visible", true);
        var button = findChild(optionsDialog, "cancelDownloadButton");
        clickItem(button);
        tryCompare(this, "optionsDialog", null);
    }

    function test_picker() {
        verify(TestContext.runContentHubPeerHookWrapper());
        browse_to("test.pdf", false);
        optionsDialog =
            waitForChild(window.contentItem, "downloadOptionsDialog");
        tryCompare(optionsDialog, "visible", true);
        var button = findChild(optionsDialog, "chooseAppButton");
        clickItem(button);
        var picker = findChild(window.contentItem, "contentPeerPicker");
        tryCompare(picker, "visible", true);
        // The following uses content-hub internals and as such is not robust
        var appGridView = waitForChild(picker, "responsiveGridViewGrid");
        verify(waitFor(function() { return (appGridView.count > 0); }));
    }

    function test_download() {
        browse_to("test.pdf", false);
        optionsDialog =
            waitForChild(window.contentItem, "downloadOptionsDialog");
        tryCompare(optionsDialog, "visible", true);
        var button = findChild(optionsDialog, "downloadFileButton");
        clickItem(button);
        downloadsPage = waitForChild(window.contentItem, "downloadsPage");
        verify(downloadsPage != null);
        tryCompare(downloadsPage, "visible", true);
    }

    function test_private_download() {
        var publicWindow = app.allWindows[0];
        verify(!publicWindow.incognito);
        open_drawer("newprivatewindow");
        tryCompare(app.allWindows, "length", 2);
        var privateWindow = app.allWindows[1];
        verify(privateWindow.incognito);
        tryCompare(privateWindow, "active", true);

        // Download pdf in private window
        window = privateWindow;
        browse_to("test.pdf", false);
        optionsDialog =
            waitForChild(window.contentItem, "downloadOptionsDialog");
        tryCompare(optionsDialog, "visible", true);
        var button = findChild(optionsDialog, "downloadFileButton");
        clickItem(button);
        downloadsPage = waitForChild(window.contentItem, "downloadsPage");
        verify(downloadsPage != null);
        tryCompare(downloadsPage, "visible", true);
        var downloadsListView = findChild(downloadsPage, "downloadsListView");
        tryCompare(downloadsListView, "count", 1);
        var delegate = waitForChild(downloadsListView, "downloadDelegate_0");
        compare(delegate.url, baseUrl + "test.pdf");
        verify(delegate.incognito);

        // Open downloads page in public window, and check that the private
        // download is not there
        window = publicWindow;
        window.requestActivate();
        tryCompare(window, "active", true);
        open_downloads();
        downloadsListView = findChild(downloadsPage, "downloadsListView");
        tryCompare(downloadsListView, "count", 0);

        // Clean up
        window = privateWindow;
        window.close();
        tryCompare(this, "window", null);
        window = publicWindow;
    }
}
