/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "FindInPage"

    function activate_find_in_page() {
        open_drawer("findinpage");
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "findInPageMode", true);
    }

    function test_activation() {
        browse_to("findinpage.html", true);
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        var findNextButton = findChild(chrome, "findNextButton");
        var findPrevButton = findChild(chrome, "findPreviousButton");
        var counter = findChild(chrome, "findInPageCounter");
        var backButton = findChild(chrome, "backButton");
        var forwardButton = findChild(chrome, "forwardButton");
        verify(!chrome.findInPageMode);
        verify(!findNextButton.visible);
        verify(!findPrevButton.visible);
        verify(!counter.visible);
        var text = chrome.text;

        activate_find_in_page();
        tryCompare(findNextButton, "visible", true);
        tryCompare(findPrevButton, "visible", true);
        tryCompare(counter, "visible", true);
        tryCompare(addressbar, "activeFocus", true);
        verify(backButton.enabled);
        verify(!forwardButton.enabled);
        tryCompare(chrome, "text", "");

        clickItem(backButton);
        tryCompare(chrome, "findInPageMode", false);
        tryCompare(findNextButton, "visible", false);
        tryCompare(findPrevButton, "visible", false);
        tryCompare(counter, "visible", false);
        tryCompare(addressbar, "activeFocus", false);
        tryCompare(chrome, "text", text);
    }

    function test_counter() {
        browse_to("findinpage.html", true);
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        var clearButton = findChild(addressbar, "clear_button");
        var counter = findChild(chrome, "findInPageCounter");
        activate_find_in_page();

        typeString("text");
        tryCompare(counter, "current", 1);
        tryCompare(counter, "count", 2);

        clickItem(clearButton);
        tryCompare(counter, "current", 0);
        tryCompare(counter, "count", 0);

        typeString("hello");
        tryCompare(counter, "current", 1);
        tryCompare(counter, "count", 1);
    }

    function test_navigation() {
        browse_to("findinpage.html", true);
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        var clearButton = findChild(addressbar, "clear_button");
        var findNextButton = findChild(chrome, "findNextButton");
        var findPrevButton = findChild(chrome, "findPreviousButton");
        var counter = findChild(chrome, "findInPageCounter");

        activate_find_in_page();
        tryCompare(findNextButton, "enabled", false);
        tryCompare(findPrevButton, "enabled", false);

        typeString("text");
        tryCompare(findNextButton, "enabled", true);
        tryCompare(findPrevButton, "enabled", true);

        clickItem(findNextButton);
        tryCompare(counter, "current", 2);
        tryCompare(counter, "count", 2);
        tryCompare(findNextButton, "enabled", true);
        clickItem(findNextButton);
        tryCompare(counter, "current", 1);
        tryCompare(counter, "count", 2);
        tryCompare(findPrevButton, "enabled", true);
        clickItem(findPrevButton);
        tryCompare(counter, "current", 2);
        tryCompare(counter, "count", 2);
        tryCompare(findNextButton, "enabled", true);
        clickItem(findNextButton);
        tryCompare(counter, "current", 1);
        tryCompare(counter, "count", 2);

        clickItem(clearButton);
        typeString("hello");
        tryCompare(findNextButton, "enabled", false);
        tryCompare(findPrevButton, "enabled", false);
    }

    function test_navigation_exits_findinpage_mode_data() {
        return [
            {path: "link.html"},
            {path: "blanktargetlink.html"},
        ];
    }

    function test_navigation_exits_findinpage_mode(data) {
        browse_to(data.path, true);
        activate_find_in_page();
        clickItem(window.currentWebview);
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "findInPageMode", false);
    }

    function test_opening_new_tab_exits_findinpage_mode() {
        activate_find_in_page();
        open_new_tab();
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "findInPageMode", false);
    }

    function test_opening_a_view_exits_findinpage_mode_data() {
        return [
            {action: "history", view: "historyView"},
            {action: "settings", view: "settingsPage"},
            {action: "downloads", view: "downloadsPage"},
            {action: "bookmarks", view: "bookmarksView"},
        ];
    }

    function test_opening_a_view_exits_findinpage_mode(data) {
        activate_find_in_page();
        open_drawer(data.action);
        verify(waitForChild(window.contentItem, data.view));
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "findInPageMode", false);
    }

    function test_find_in_page_not_in_menu_in_new_tab() {
        open_new_tab();
        var drawer = open_drawer(null);
        var findAction = findChild(drawer, "findinpage");
        verify(!findAction.visible);
    }

    function test_focus_on_enter() {
        // Regression test for https://launchpad.net/bugs/1508130
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        activate_find_in_page();
        tryCompare(addressbar, "activeFocus", true);
        var backButton = findChild(chrome, "backButton");
        clickItem(backButton);
        tryCompare(chrome, "findInPageMode", false);
        activate_find_in_page();
        tryCompare(addressbar, "activeFocus", true);
    }
}
