/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "History"

    property var historyView: null
    property var expandedView: null

    function open_history() {
        open_drawer("history");
        historyView = waitForChild(window.contentItem, "historyView");
        verify(historyView != null);
        tryCompare(historyView, "activeFocus", true);
    }

    function expect_history_entries(ordered_urls) {
        var i = 0;
        var entries = [];
        var entry = null;
        if (window.wide) {
            var listview = findChild(historyView, "urlsListView");
            tryCompare(listview, "count", ordered_urls.length);
            for (i in ordered_urls) {
                entries.push(waitForChild(listview, "historyDelegate_%1".arg(i)));
            }
        } else {
            var domainsListview = findChild(historyView, "domainsListView");
            tryCompare(domainsListview, "count", 1);
            var domainEntry = findChild(domainsListview, "historyViewDomainDelegate");
            clickItem(domainEntry);
            expandedView = waitForChild(window.contentItem, "expandedHistoryView");
            verify(expandedView != null);
            tryCompare(expandedView, "visible", true);
            tryCompare(expandedView, "count", ordered_urls.length);
            for (i in ordered_urls) {
                entries.push(waitForChild(expandedView, "entriesDelegate_%1".arg(i)));
            }
        }
        compare(entries.length, ordered_urls.length);
        for (i in entries) {
            entry = entries[i];
            verify(entry != null);
            compare(entry.url, ordered_urls[i]);
        }
        return entries;
    }

    function test_404_not_saved() {
        browse_to("notfound.html", true);
        // A valid url to be sure the fact the 404 page isn't present in the
        // history view isn't a timing issue.
        browse_to("test2.html", true);
        open_history();
        expect_history_entries([baseUrl + "test2.html", baseUrl + "test1.html"]);
    }

    function test_expanded_history_view_header_swallows_clicks() {
        // Regression test for https://launchpad.net/bugs/1518904
        if (window.wide) {
            skip("Only on narrow form factors");
        }
        open_history();
        var domainsListview = findChild(historyView, "domainsListView");
        tryCompare(domainsListview, "count", 1);
        var domainEntry = findChild(domainsListview, "historyViewDomainDelegate");
        clickItem(domainEntry);
        expandedView = waitForChild(window.contentItem, "expandedHistoryView");
        verify(expandedView != null);
        tryCompare(expandedView, "visible", true);
        mouseClick(domainEntry, domainEntry.width / 2, 1);
        wait(500);
        // There should be only one instance on the expanded history view.
        var doneButton = findChild(expandedView, "doneButton");
        clickItem(doneButton);
        tryCompare(this, "expandedView", null);
    }

    function test_favicon_saved() {
        // Regression test for https://launchpad.net/bugs/1549780
        browse_to("favicon.html", true);
        open_history();
        var url = baseUrl + "favicon.html";
        var entry = expect_history_entries([url, baseUrl + "test1.html"])[0];
        compare(entry.url, url);
        compare(entry.icon, baseUrl + "assets/icon1.png");
    }

    function test_favicon_updated() {
        // Verify that a page dynamically updating its favicon
        // triggers an update in the history database.
        browse_to("changingfavicon.html", true);
        open_history();
        var url = baseUrl + "changingfavicon.html";
        var entry = expect_history_entries([url, baseUrl + "test1.html"])[0];
        var icons = [];
        while (icons.length < 3) {
            compare(entry.url, url);
            var icon = entry.icon.toString();
            compare(icon.indexOf(baseUrl), 0);
            compare(icon.lastIndexOf(".png"), icon.length - 4);
            if (icons.indexOf(icon) == -1) {
                icons.push(icon);
            }
            wait(100);
        }
    }

    function test_title_saved() {
        open_history();
        var entry = expect_history_entries([baseUrl + "test1.html"])[0];
        compare(entry.title, "test page 1");
    }

    function test_title_not_updated() {
        // Verify that a page dynamically updating its title
        // does NOT trigger an update in the history database.
        browse_to("changingtitle.html", true);
        open_history();
        var url = baseUrl + "changingtitle.html";
        var entry = expect_history_entries([url, baseUrl + "test1.html"])[0];
        for (var i = 0; i < 10; ++i) {
            compare(entry.title, "title0");
            wait(500);
        }
    }

    function test_pushstate_updates_history() {
        browse_to("pushstate.html", true);
        clickItem(window.currentWebview);
        var pushed = baseUrl + "statepushed.html";
        tryCompare(window.currentWebview, "url", pushed);
        tryCompare(window.currentWebview, "loading", false);
        open_history();
        expect_history_entries([pushed, baseUrl + "pushstate.html", baseUrl + "test1.html"]);
    }

    function test_private_window_no_history() {
        var publicWindow = app.allWindows[0];
        verify(!publicWindow.incognito);
        open_drawer("newprivatewindow");
        tryCompare(app.allWindows, "length", 2);
        var privateWindow = app.allWindows[1];
        verify(privateWindow.incognito);
        tryCompare(privateWindow, "active", true);

        // Open URL in private window
        window = privateWindow;
        browse_to("test2.html", true);

        // Open history view in public window and check that URL is not there
        window = publicWindow;
        window.requestActivate();
        tryCompare(window, "active", true);
        open_history();
        expect_history_entries([baseUrl + "test1.html"]);

        // Clean up
        window = privateWindow;
        window.close();
        tryCompare(this, "window", null);
        window = publicWindow;
    }

    function test_title_correct_redirect_header() {
        // Regression test for https://launchpad.net/bugs/1603835
        browse_to("redirect-no-title-header.py", true);
        open_history();
        var destination = baseUrl + "redirect-destination.html";
        var entries = expect_history_entries([destination, baseUrl + "test1.html"]);
        compare(entries[0].title, destination.substr("http://".length));
        compare(entries[1].title, "test page 1");
    }

    function test_title_correct_redirect_js() {
        // Regression test for https://launchpad.net/bugs/1603835
        var redirect = baseUrl + "redirect-no-title-js.html";
        browse_to("redirect-no-title-js.html", true);
        open_history();
        var destination = baseUrl + "redirect-destination.html";
        var entries = expect_history_entries([destination,
                                              baseUrl + "redirect-no-title-js.html",
                                              baseUrl + "test1.html"]);
        compare(entries[0].title, destination.substr("http://".length));
        compare(entries[1].title, redirect.substr("http://".length));
        compare(entries[2].title, "test page 1");
    }
}
