/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import QtTest 1.0

SingleWindowTestCase {
    name: "Keyboard"

    property var bookmarksView: null
    property var historyView: null
    property var settingsPage: null
    property var window2: null

    bookmarksDbFile: Qt.resolvedUrl("keyboard-bookmarks.sqlite")

    SignalSpy {
        id: loadingStateSpy
        target: window ? window.currentWebview : null
        signalName: "loadingStateChanged"
    }

    function open_tab(path) {
        var count = window.tabsModel.count;
        keyClick(Qt.Key_T, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", count + 1);
        tryCompare(window.tabsModel, "currentIndex", window.wide ? count : 0);
        waitFor(function() { return (window.currentWebview != null); });
        verify(waitForChild(window.contentItem, "newTabView"));
        var newTabView = findChild(window.contentItem, "newTabView");
        waitForRendering(newTabView);
        // Give some time for the active focus to settle
        // (it depends on the presence of a physical keyboard).
        // XXX: this is racy at best.
        wait(100);
        if (path) {
            browse_to(path, true);
        }
    }

    function open_tabs(number) {
        for (var i = 1; i <= number; ++i) {
            open_tab("test1.html?index=%1".arg(i));
        }
    }

    function check_tab_number(index) {
        var url = baseUrl + "test1.html" + ((index > 0) ? "?index=%1".arg(index) : "");
        check_current_url(url);
    }

    function test_new_tab() {
        compare(window.tabsModel.count, 1);
        keyClick(Qt.Key_T, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 2);
        check_current_url("");
        verify(waitForChild(window.contentItem, "newTabView"));
    }

    function test_switch_tabs() {
        open_tabs(2);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_tab_number(0);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_tab_number(1);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_tab_number(2);
        keyClick(Qt.Key_PageDown, Qt.ControlModifier);
        check_tab_number(0);
        keyClick(Qt.Key_Tab, Qt.ControlModifier | Qt.ShiftModifier);
        check_tab_number(window.wide ? 2 : 1);
        keyClick(Qt.Key_PageUp, Qt.ControlModifier);
        check_tab_number(window.wide ? 1 : 2);
    }

    function test_can_switch_tabs_after_suggestions_escape() {
        open_tabs(1);
        var suggestions = findChild(window.contentItem, "suggestionsList");
        focus_address_bar(true);
        typeString("el");
        tryCompare(suggestions, "opacity", 1.0);
        keyClick(Qt.Key_Down);
        tryCompare(suggestions, "activeFocus", true);
        keyClick(Qt.Key_Escape);
        tryCompare(suggestions, "opacity", 0.0);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_tab_number(0);
    }

    function test_switch_tabs_from_tabs_view() {
        if (window.wide) {
            skip("Only on narrow form factors");
        }
        open_tabs(1);
        var tabsView = open_tabs_view();
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_tab_number(0);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_tab_number(1);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_tab_number(0);
        keyClick(Qt.Key_Escape);
        tryCompare(tabsView, "visible", false);
        check_tab_number(0);
    }

    function test_close_tabs_data() {
        return [
            { key: Qt.Key_F4, modifiers: Qt.ControlModifier },
            { key: Qt.Key_W, modifiers: Qt.ControlModifier },
        ];
    }

    function test_close_tabs(data) {
        open_tabs(1);
        keyClick(data.key, data.modifiers);
        tryCompare(window.tabsModel, "count", 1);
        check_tab_number(0);
        if (!window.wide) {
            // XXX: in wide mode, closing the last open tab closes the app,
            // which is not well handled by the test (it segfaults)
            keyClick(data.key, data.modifiers);
            tryCompare(window.tabsModel, "count", 1);
            check_current_url("");
        }
    }

    function test_close_tabs_tabs_view() {
        if (window.wide) {
            skip("Only on narrow form factors");
        }
        open_tabs(1);
        var tabsView = open_tabs_view();
        keyClick(Qt.Key_W, Qt.ControlModifier);
        check_tab_number(0);
        keyClick(Qt.Key_F4, Qt.ControlModifier);
        tryCompare(tabsView, "visible", false);
        check_current_url("");
    }

    function test_select_address_bar_data() {
        return [
            { key: Qt.Key_L, modifiers: Qt.ControlModifier },
            { key: Qt.Key_D, modifiers: Qt.AltModifier },
            { key: Qt.Key_F6, modifiers: Qt.NoModifier },
        ];
    }

    function test_select_address_bar(data) {
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        verify(!addressbar.activeFocus);
        keyClick(data.key, data.modifiers);
        tryCompare(addressbar, "activeFocus", true);
    }

    function test_escape_from_address_bar() {
        keyClick(Qt.Key_D, Qt.AltModifier);
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        tryCompare(addressbar, "selectedText", addressbar.text);
        keyClick(Qt.Key_Escape);
        tryCompare(addressbar, "selectedText", "");
        tryCompare(addressbar, "activeFocus", false);
        tryCompare(window.currentWebview, "activeFocus", true);
    }

    function test_reload_data() {
        return [
            { key: Qt.Key_R, modifiers: Qt.ControlModifier },
            { key: Qt.Key_F5, modifiers: Qt.NoModifier },
        ];
    }

    function test_reload(data) {
        loadingStateSpy.clear();
        keyClick(data.key, data.modifiers);
        loadingStateSpy.wait();
        tryCompare(window.currentWebview, "loading", false);
    }

    function test_bookmark() {
        var chrome = findChild(window.contentItem, "chrome");
        verify(!chrome.bookmarked);
        keyClick(Qt.Key_D, Qt.ControlModifier);
        tryCompare(chrome, "bookmarked", true);
        keyClick(Qt.Key_Escape);
        tryCompare(chrome, "bookmarked", false);
        keyClick(Qt.Key_D, Qt.ControlModifier);
        tryCompare(chrome, "bookmarked", true);
        keyClick(Qt.Key_D, Qt.ControlModifier);
        tryCompare(chrome, "bookmarked", false);
    }

    function test_cannot_bookmark_empty_tab() {
        open_tab("");
        keyClick(Qt.Key_D, Qt.ControlModifier);
        verify(!waitForChild(window.contentItem, "bookmarkOptions", 500));
    }

    function test_history_navigation_data() {
        return [
            { prevKey: Qt.Key_Left, prevModifier: Qt.AltModifier,
              nextKey: Qt.Key_Right, nextModifier: Qt.AltModifier },
            { prevKey: Qt.Key_Backspace, prevModifier: Qt.NoModifier,
              nextKey: Qt.Key_Backspace, nextModifier: Qt.ShiftModifier },
        ];
    }

    function test_history_navigation(data) {
        var previous = window.currentWebview.url;
        browse_to("test2.html", true);
        var next = window.currentWebview.url;
        keyClick(data.prevKey, data.prevModifier);
        tryCompare(window.currentWebview, "url", previous);
        keyClick(data.nextKey, data.nextModifier);
        tryCompare(window.currentWebview, "url", next);
    }

    function test_backspace_does_not_go_back_when_html_text_field_focused() {
        // Regression test for https://launchpad.net/bugs/1569938
        browse_to("textarea.html", true);
        loadingStateSpy.clear();
        clickItem(window.currentWebview);
        tryCompare(window.currentWebview, "activeFocus", true);
        keyClick(Qt.Key_Backspace);
        wait(500);
        compare(loadingStateSpy.count, 0);
    }

    function test_toggle_view_data() {
        return [
            { view: "bookmarksView", key: Qt.Key_O, modifiers: Qt.ControlModifier | Qt.ShiftModifier },
            { view: "historyView", key: Qt.Key_H, modifiers: Qt.ControlModifier },
        ];
    }

    function test_toggle_view(data) {
        compare(findChild(window.contentItem, data.view), null);
        keyClick(data.key, data.modifiers);
        this[data.view] = waitForChild(window.contentItem, data.view);
        verify(this[data.view] != null);
        tryCompare(this[data.view], "activeFocus", true);
        waitForRendering(this[data.view]);
        keyClick(Qt.Key_Escape);
        tryCompare(this, data.view, null);
        tryCompare(window.currentWebview, "activeFocus", true);
    }

    function test_toggle_view_from_menu_data() {
        return [
            { action: "bookmarks", view: "bookmarksView" },
            { action: "history", view: "historyView" },
        ];
    }

    function test_toggle_view_from_menu(data) {
        compare(findChild(window.contentItem, data.view), null);
        open_drawer(data.action);
        this[data.view] = waitForChild(window.contentItem, data.view);
        verify(this[data.view] != null);
        tryCompare(this[data.view], "activeFocus", true);
        waitForRendering(this[data.view]);
        keyClick(Qt.Key_Escape);
        tryCompare(this, data.view, null);
        tryCompare(window.currentWebview, "activeFocus", true);
    }

    function test_new_tab_from_view_data() {
        return test_toggle_view_from_menu_data();
    }

    function test_new_tab_from_view(data) {
        compare(findChild(window.contentItem, data.view), null);
        open_drawer(data.action);
        this[data.view] = waitForChild(window.contentItem, data.view);
        verify(this[data.view] != null);
        tryCompare(this[data.view], "activeFocus", true);
        keyClick(Qt.Key_T, Qt.ControlModifier);
        tryCompare(this, data.view, null);
        tryCompare(window.tabsModel, "count", 2);
        verify(waitForChild(window.contentItem, "newTabView") != null);
    }

    function test_search_in_history() {
        if (!window.wide) {
            skip("Only on wide form factors");
        }
        compare(findChild(window.contentItem, "historyView"), null);
        keyClick(Qt.Key_H, Qt.ControlModifier);
        historyView = waitForChild(window.contentItem, "historyView");
        tryCompare(historyView, "activeFocus", true);
        verify(!historyView.searchMode);
        keyClick(Qt.Key_F, Qt.ControlModifier);
        tryCompare(historyView, "searchMode", true);
        var searchField = findChild(historyView, "searchQuery");
        tryCompare(searchField, "visible", true);
        tryCompare(searchField, "activeFocus", true);
        keyClick(Qt.Key_Escape);
        tryCompare(historyView, "searchMode", false);
        tryCompare(searchField, "parent", null);
        keyClick(Qt.Key_Escape);
        tryCompare(this, "historyView", null);
    }

    function test_opening_a_view_exits_findinpage_mode_data() {
        return test_toggle_view_data();
    }

    function test_opening_a_view_exits_findinpage_mode(data) {
        keyClick(Qt.Key_F, Qt.ControlModifier);
        var chrome = findChild(window.contentItem, "chrome");
        tryCompare(chrome, "findInPageMode", true);
        var addressbar = findChild(chrome, "addressBarTextField");
        tryCompare(addressbar, "activeFocus", true);
        keyClick(data.key, data.modifiers);
        this[data.view] = waitForChild(window.contentItem, data.view);
        verify(this[data.view] != null);
        tryCompare(this[data.view], "activeFocus", true);
        tryCompare(chrome, "findInPageMode", false);
        tryCompare(addressbar, "activeFocus", false);
        keyClick(Qt.Key_Escape);
        tryCompare(this, data.view, null);
    }

    function test_escape_settings() {
        open_drawer("settings");
        settingsPage = waitForChild(window.contentItem, "settingsPage");
        verify(settingsPage != null);
        tryCompare(settingsPage, "activeFocus", true);
        waitForRendering(settingsPage);
        keyClick(Qt.Key_Escape);
        tryCompare(this, "settingsPage", null);
        tryCompare(window.currentWebview, "activeFocus", true);
    }

    function test_find_in_page_ctrl_f() {
        var chrome = findChild(window.contentItem, "chrome");
        verify(!chrome.findInPageMode);
        keyClick(Qt.Key_F, Qt.ControlModifier);
        tryCompare(chrome, "findInPageMode", true);
        keyClick(Qt.Key_Escape);
        tryCompare(chrome, "findInPageMode", false);

        keyClick(Qt.Key_T, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 2);
        check_current_url("");
        verify(waitForChild(window.contentItem, "newTabView"));
        wait(100);
        verify(!chrome.findInPageMode);
    }

    function test_find_previous_and_next() {
        browse_to("findinpage.html", true);
        var chrome = findChild(window.contentItem, "chrome");
        verify(!chrome.findInPageMode);
        keyClick(Qt.Key_F, Qt.ControlModifier);
        tryCompare(chrome, "findInPageMode", true);
        var addressbar = findChild(chrome, "addressBarTextField");
        tryCompare(addressbar, "activeFocus", true);
        typeString("e");
        var counter = findChild(chrome, "findInPageCounter");
        tryCompare(counter, "count", 4);
        tryCompare(counter, "current", 1);
        var indexes = [2, 3, 4, 1, 2, 3, 4, 1, 2];
        for (var i in indexes) {
            keyClick(Qt.Key_G, Qt.ControlModifier);
            tryCompare(counter, "current", indexes[i]);
        }
        indexes = [1, 4, 3, 2, 1, 4, 3, 2];
        for (var j in indexes) {
            keyClick(Qt.Key_G, Qt.ControlModifier | Qt.ShiftModifier);
            tryCompare(counter, "current", indexes[j]);
        }
    }

    function test_navigate_between_address_bar_and_new_tab_view() {
        if (!window.wide) {
            skip("Only on wide form factors");
        }
        keyClick(Qt.Key_T, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 2);
        check_current_url("");
        var newTabView = waitForChild(window.contentItem, "newTabView");
        verify(newTabView != null);
        waitForRendering(newTabView);
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        tryCompare(addressbar, "activeFocus", true);
        keyClick(Qt.Key_Down);
        tryCompare(addressbar, "activeFocus", false);
        tryCompare(newTabView, "activeFocus", true);
        keyClick(Qt.Key_Up);
        tryCompare(addressbar, "activeFocus", true);
        tryCompare(newTabView, "activeFocus", false);
    }

    function test_ctrl_w_closes_tabs_only_from_main_view_data() {
        return [
            { action: "bookmarks", view: "bookmarksView" },
            { action: "history", view: "historyView" },
            { action: "settings", view: "settingsPage" },
        ];
    }

    function test_ctrl_w_closes_tabs_only_from_main_view(data) {
        // Verify that Ctrl+w doesn’t work when either the history
        // view, the bookmarks view or the settings view are shown
        // (https://launchpad.net/bugs/1524353).
        keyClick(Qt.Key_T, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 2);
        open_drawer(data.action);
        this[data.view] = waitForChild(window.contentItem, data.view);
        verify(this[data.view] != null);
        tryCompare(this[data.view], "activeFocus", true);
        waitForRendering(this[data.view]);
        keyClick(Qt.Key_W, Qt.ControlModifier);
        keyClick(Qt.Key_Escape);
        tryCompare(this, data.view, null);
        compare(window.tabsModel.count, 2);
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 1);
    }

    function test_addressbar_cleared_when_opening_new_tab() {
        // Verify that when opening a new tab while the address bar was
        // focused, the address bar is cleared.
        keyClick(Qt.Key_L, Qt.ControlModifier);
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        tryCompare(addressbar, "activeFocus", true);
        compare(addressbar.text, window.currentWebview.url);
        keyClick(Qt.Key_T, Qt.ControlModifier);
        tryCompare(addressbar, "text", "");
    }

    function test_addressbar_cleared_when_switching_between_new_tabs() {
        // Verify that when opening a new tab while a new tab was already open
        // with text input in the address bar, the address bar is cleared.
        keyClick(Qt.Key_T, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 2);
        check_current_url("");
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        tryCompare(addressbar, "activeFocus", true);
        tryCompare(addressbar, "text", "");
        typeString("abc");
        tryCompare(addressbar, "text", "abc");
        keyClick(Qt.Key_T, Qt.ControlModifier);
        tryCompare(addressbar, "text", "");
    }

    function expect_zoom(expected) {
        fuzzyCompare(window.currentWebview.zoomFactor, expected, 0.0001);
    }

    function test_zoom_in_and_out() {
        window.currentWebview.zoomFactor = 1.0;
        expect_zoom(1.0);
        var zooms = [1.1, 1.25, 1.5, 1.75, 2.0, 2.5, 3.0, 4.0, 5.0];
        for (var i in zooms) {
            keyClick(Qt.Key_Plus, Qt.ControlModifier);
            expect_zoom(zooms[i]);
        }
        keyClick(Qt.Key_0, Qt.ControlModifier);
        expect_zoom(1.0);
        zooms = [0.9, 0.75, 0.666, 0.5, 0.333, 0.25];
        for (var j in zooms) {
            keyClick(Qt.Key_Minus, Qt.ControlModifier);
            expect_zoom(zooms[j]);
        }
        keyClick(Qt.Key_0, Qt.ControlModifier);
        expect_zoom(1.0);
    }

    function test_zoom_qwerty_compatibility_shortcuts() {
        // Ref: https://launchpad.net/bugs/1624381
        window.currentWebview.zoomFactor = 1.0;
        expect_zoom(1.0);
        keyClick(Qt.Key_Equal, Qt.ControlModifier);
        expect_zoom(1.1);
        keyClick(Qt.Key_Underscore, Qt.ControlModifier);
        expect_zoom(1.0);
    }

    function test_zoom_affects_domain() {
        window.currentWebview.zoomFactor = 1.0;
        open_tabs(1);
        keyClick(Qt.Key_Plus, Qt.ControlModifier);
        expect_zoom(1.1);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_tab_number(0);
        expect_zoom(1.1);
    }

    function test_open_new_window_data() {
        return [
            { incognito: false, key: Qt.Key_N, modifiers: Qt.ControlModifier },
            { incognito: true, key: Qt.Key_N, modifiers: Qt.ControlModifier | Qt.ShiftModifier },
        ];
    }

    function test_open_new_window(data) {
        compare(app.allWindows.length, 1);
        keyClick(data.key, data.modifiers);
        tryCompare(app.allWindows, "length", 2);
        window2 = app.allWindows[1];
        compare(window2.incognito, data.incognito);
        tryCompare(window2, "visible", true);
        waitFor(function() { return (window2.currentWebview != null); });
        window2.close();
        tryCompare(this, "window2", null);
    }

    function test_undo_close_tab_data() {
        return [
            { key: Qt.Key_W, modifiers: Qt.ControlModifier | Qt.ShiftModifier },
            { key: Qt.Key_T, modifiers: Qt.ControlModifier | Qt.ShiftModifier },
        ];
    }

    function test_undo_close_tab(data) {
        open_tab("test1.html?index=1");
        // Insert a "new tab view" page in the middle, without any page loaded
        // so that we can verify that it will not be restored.
        open_new_tab();
        open_tab("test1.html?index=2");
        tryCompare(window.tabsModel, "count", 4);
        for (var i = 3; i > 0; --i) {
            keyClick(Qt.Key_W, Qt.ControlModifier);
            tryCompare(window.tabsModel, "count", i);
        }
        check_tab_number(0);
        for (var j = 1; j < 3; ++j) {
            keyClick(data.key, data.modifiers);
            tryCompare(window.tabsModel, "count", j + 1);
            check_tab_number(j);
        }
        keyClick(data.key, data.modifiers);
        compare(window.tabsModel.count, 3);
        check_tab_number(2);
    }

    function test_focus_on_switch() {
        // Test that switching between tabs correctly resets focus to the
        // webview if a page is loaded, and to the address bar if we are in
        // the new page view.
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        verify(!addressbar.activeFocus);
        open_tab("");
        tryCompare(addressbar, "activeFocus", true);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        tryCompare(addressbar, "activeFocus", false);
        tryCompare(window.currentWebview, "activeFocus", true);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        tryCompare(addressbar, "activeFocus", true);
        tryCompare(window.currentWebview, "activeFocus", false);
    }

    function test_focus_on_close() {
        // Test that closing tabs correctly resets focus,
        // allowing keyboard shortcuts to work without interruption.
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        verify(!addressbar.activeFocus);
        open_tab("");
        open_tab("test1.html");
        open_tab("test2.html");
        open_tab("");
        open_tab("");
        tryCompare(window.tabsModel, "count", 6);
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 5);
        if (window.wide) {
            tryCompare(addressbar, "activeFocus", true);
        }
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 4);
        tryCompare(window.currentWebview, "activeFocus", true);
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 3);
        tryCompare(window.currentWebview, "activeFocus", true);
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 2);
        if (window.wide) {
            tryCompare(addressbar, "activeFocus", true);
        }
    }
}
