/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import QtTest 1.0

SingleWindowTestCase {
    name: "NewTabViewContents"

    bookmarksDbFile: Qt.resolvedUrl("newtabviewcontents-bookmarks.sqlite")
    configFile: Qt.resolvedUrl("newtabviewcontents-config.conf")

    property var newTabView: null
    readonly property string bookmarksRoot:
        "http://www.rsc.org/periodic-table/element"

    function narrow_init() {
        if (window.wide) {
            skip("Only on narrow form factors");
        }
        open_new_tab();
        newTabView = findChild(window.contentItem, "newTabView");
        verify(newTabView != null);
    }

    function narrow_toggle_bookmark_folder(name) {
        var folders = findChild(newTabView, "bookmarksFolderListView");
        var folder = findChild(folders, "bookmarkFolderDelegate_%1".arg(name));
        var header = findChild(folder, "bookmarkFolderHeader");
        clickItem(header);
    }

    function narrow_remove_first_bookmark_from_folder(name, url) {
        var folders = findChild(newTabView, "bookmarksFolderListView");
        var folder = findChild(folders, "bookmarkFolderDelegate_%1".arg(name));
        var bookmarksInFolder = findChild(folder, "bookmarksInFolder");
        var bookmark = waitForChild(bookmarksInFolder, "urlDelegate_0");
        compare(bookmark.url, url);
        swipeToDeleteAndConfirm(bookmark);
    }

    function test_narrow_default_home_bookmark() {
        narrow_init();
        var homepageBookmark = findChild(newTabView, "homepageBookmark");
        compare(homepageBookmark.url, "http://test/test2");
        clickItem(homepageBookmark);
        tryCompare(this, "newTabView", null);
        check_current_url("http://test/test2");
    }

    function test_narrow_open_top_site() {
        narrow_init();
        var topSites = findChild(newTabView, "topSitesList");
        compare(topSites.count, 1);
        var topSite = findChild(topSites, "topSiteItem");
        var url = baseUrl + "test1.html";
        compare(topSite.url, url);
        clickItem(topSite);
        tryCompare(this, "newTabView", null);
        check_current_url(url);
        tryCompare(window.currentWebview, "loading", false);
    }

    function test_narrow_open_bookmark() {
        narrow_init();
        var bookmarksList = findChild(newTabView, "bookmarksList");
        var bookmark = findChild(bookmarksList, "bookmark_2");
        var url = "http://www.rsc.org/periodic-table/element/24/chromium";
        compare(bookmark.url, url);
        clickItem(bookmark);
        tryCompare(this, "newTabView", null);
        check_current_url(url);
    }

    function test_narrow_open_bookmark_when_expanded() {
        narrow_init();
        var moreButton = findChild(newTabView, "bookmarks.moreButton");
        verify(moreButton.visible);
        clickItem(moreButton);
        var folders = findChild(newTabView, "bookmarksFolderListView");
        var folderDelegate = findChild(folders, "bookmarkFolderDelegate_");
        var bookmarksInFolder = findChild(folderDelegate, "bookmarksInFolder");
        compare(bookmarksInFolder.count, 5);
        var bookmark = waitForChild(bookmarksInFolder, "urlDelegate_3");
        var url = "http://www.rsc.org/periodic-table/element/77/iridium";
        compare(bookmark.url, url);
        clickItem(bookmark);
        tryCompare(this, "newTabView", null);
        check_current_url(url);
    }

    function test_narrow_bookmarks_section_expands_and_collapses() {
        narrow_init();
        var bookmarksList = findChild(newTabView, "bookmarksList");
        var topSites = findChild(newTabView, "topSitesList");
        verify(topSites.visible);
        // When the bookmarks list is collapsed, it shows a maximum of 5 entries
        compare(bookmarksList.count, 5);
        // When expanded, it shows all entries
        var moreButton = findChild(newTabView, "bookmarks.moreButton");
        verify(moreButton.visible);
        clickItem(moreButton);
        var folders = findChild(newTabView, "bookmarksFolderListView");
        var folderDelegate = findChild(folders, "bookmarkFolderDelegate_");
        var bookmarksInFolder = findChild(folderDelegate, "bookmarksInFolder");
        compare(bookmarksInFolder.count, 5);
        verify(!topSites.visible);
        // Collapse again
        verify(moreButton.visible);
        clickItem(moreButton);
        bookmarksList = findChild(newTabView, "bookmarksList");
        compare(bookmarksList.count, 5);
        verify(topSites.visible);
    }

    function test_narrow_remove_bookmarks_when_collapsed() {
        narrow_init();
        var bookmarksList = findChild(newTabView, "bookmarksList");
        compare(bookmarksList.count, 5);
        var moreButton = findChild(newTabView, "bookmarks.moreButton");
        verify(moreButton.visible);

        var bookmark = findChild(bookmarksList, "bookmark_1");
        compare(bookmark.url, "%1/31/gallium".arg(bookmarksRoot));
        swipeToDeleteAndConfirm(bookmark);
        tryCompare(bookmark, "url", "%1/24/chromium".arg(bookmarksRoot));
        verify(moreButton.visible);
        compare(bookmarksList.count, 5);

        swipeToDeleteAndConfirm(bookmark);
        tryCompare(bookmark, "url", "%1/77/iridium".arg(bookmarksRoot));
        verify(!moreButton.visible);
        compare(bookmarksList.count, 5);

        swipeToDeleteAndConfirm(bookmark);
        tryCompare(moreButton, "visible", false);
        tryCompare(bookmarksList, "count", 4);
        bookmark = findChild(bookmarksList, "bookmark_1");
        compare(bookmark.url, "%1/116/livermorium".arg(bookmarksRoot));
    }

    function test_narrow_remove_bookmarks_when_expanded() {
        narrow_init();
        var moreButton = findChild(newTabView, "bookmarks.moreButton");
        verify(moreButton.visible);
        var topSites = findChild(newTabView, "topSitesList");
        verify(topSites.visible);
        clickItem(moreButton);
        verify(!topSites.visible);
        var folders = findChild(newTabView, "bookmarksFolderListView");
        var folderDelegate = findChild(folders, "bookmarkFolderDelegate_");
        var bookmarksInFolder = findChild(folderDelegate, "bookmarksInFolder");
        compare(bookmarksInFolder.count, 5);
        narrow_toggle_bookmark_folder("Actinide");
        var url = "%1/89/actinium".arg(bookmarksRoot);
        narrow_remove_first_bookmark_from_folder("Actinide", url);
        narrow_toggle_bookmark_folder("NobleGas");
        url = "%1/2/helium".arg(bookmarksRoot);
        narrow_remove_first_bookmark_from_folder("NobleGas", url);
        tryCompare(moreButton, "visible", false);
        verify(topSites.visible);
    }

    function test_narrow_show_bookmarks_folders_when_expanded() {
        narrow_init();
        var moreButton = findChild(newTabView, "bookmarks.moreButton");
        verify(moreButton.visible);
        clickItem(moreButton);
        var folders = findChild(newTabView, "bookmarksFolderListView");
        compare(folders.count, 3);

        var folder = findChild(folders, "bookmarkFolderDelegate_");
        var bookmarksInFolder = findChild(folder, "bookmarksInFolder");
        compare(bookmarksInFolder.count, 5);

        narrow_toggle_bookmark_folder("Actinide");
        folder = findChild(folders, "bookmarkFolderDelegate_Actinide");
        bookmarksInFolder = findChild(folder, "bookmarksInFolder");
        compare(bookmarksInFolder.count, 1);
        waitForRendering(waitForChild(bookmarksInFolder, "urlDelegate_0"));

        narrow_toggle_bookmark_folder("NobleGas");
        folder = findChild(folders, "bookmarkFolderDelegate_NobleGas");
        bookmarksInFolder = findChild(folder, "bookmarksInFolder");
        compare(bookmarksInFolder.count, 1);
    }

    function test_narrow_collapsed_bookmarks_folders_when_expanded() {
        narrow_init();
        var moreButton = findChild(newTabView, "bookmarks.moreButton");
        verify(moreButton.visible);
        clickItem(moreButton);
        var folders = findChild(newTabView, "bookmarksFolderListView");
        compare(folders.count, 3);

        var folder = findChild(folders, "bookmarkFolderDelegate_");
        var bookmarksInFolder = findChild(folder, "bookmarksInFolder");
        compare(bookmarksInFolder.count, 5);

        folder = findChild(folders, "bookmarkFolderDelegate_Actinide");
        verify(!waitForChild(folder, "bookmarksInFolder", 500));

        folder = findChild(folders, "bookmarkFolderDelegate_NobleGas");
        verify(!waitForChild(folder, "bookmarksInFolder", 500));
    }

    function test_narrow_hide_empty_bookmarks_folders_when_expanded() {
        narrow_init();
        var moreButton = findChild(newTabView, "bookmarks.moreButton");
        verify(moreButton.visible);
        clickItem(moreButton);
        var folders = findChild(newTabView, "bookmarksFolderListView");
        compare(folders.count, 3);

        narrow_toggle_bookmark_folder("Actinide");
        var url = "%1/89/actinium".arg(bookmarksRoot);
        narrow_remove_first_bookmark_from_folder("Actinide", url);
        verify(waitFor(function() {
            return !findChild(folders, "bookmarkFolderDelegate_Actinide"); }));

        verify(findChild(folders, "bookmarkFolderDelegate_"));
        verify(findChild(folders, "bookmarkFolderDelegate_NobleGas"));
    }

    function test_narrow_bookmarks_folder_expands_and_collapses() {
        narrow_init();
        var moreButton = findChild(newTabView, "bookmarks.moreButton");
        verify(moreButton.visible);
        clickItem(moreButton);
        var folders = findChild(newTabView, "bookmarksFolderListView");
        compare(folders.count, 3);

        var folder = findChild(folders, "bookmarkFolderDelegate_");
        var bookmarksInFolder = findChild(folder, "bookmarksInFolder");
        compare(bookmarksInFolder.count, 5);

        narrow_toggle_bookmark_folder("");
        verify(waitFor(function() {
            return !findChild(folder, "bookmarksInFolder"); }));

        narrow_toggle_bookmark_folder("");
        bookmarksInFolder = findChild(folder, "bookmarksInFolder");
        compare(bookmarksInFolder.count, 5);
    }

    function test_narrow_remove_top_sites() {
        narrow_init();
        var topSites = findChild(newTabView, "topSitesList");
        verify(topSites.visible);
        compare(topSites.count, 1);
        var noTopSitesLabel = findChild(newTabView, "notopsites");
        verify(!noTopSitesLabel.visible);

        var topSite = findChild(topSites, "topSiteItem");
        clickItem(topSite, Qt.RightButton);
        verify(
            waitForChild(window.contentItem, "urlPreviewDelegate.contextMenu"));
        var menu =
            findChild(window.contentItem, "urlPreviewDelegate.contextMenu");
        var deleteAction = findChild(menu, "delete_button");
        clickItem(deleteAction);
        tryCompare(noTopSitesLabel, "visible", true);
        verify(!topSites.visible);
        compare(topSites.count, 0);
    }

    function wide_init() {
        if (!window.wide) {
            skip("Only on wide form factors");
        }
        open_new_tab();
        newTabView = findChild(window.contentItem, "newTabView");
        verify(newTabView != null);
    }

    function wide_go_to_section(index) {
        var sections = findChild(newTabView, "sections");
        if (sections.selectedIndex != index) {
            clickItem(findChild(sections, "section_button_%1".arg(index)));
        }
    }

    function test_wide_remove_bookmark() {
        wide_init();
        wide_go_to_section(1);
        var bookmarksList = findChild(newTabView, "bookmarksList");
        compare(bookmarksList.count, 5);
        var bookmark = waitForChild(bookmarksList, "bookmarkItem_1");
        swipeToDeleteAndConfirm(bookmark);
        tryCompare(bookmarksList, "count", 4);
    }

    function test_wide_remove_top_sites() {
        wide_init();
        wide_go_to_section(0);
        var topSites = findChild(newTabView, "topSitesList");
        compare(topSites.count, 1);
        var topSite = findChild(topSites, "topSiteItem");
        clickItem(topSite, Qt.RightButton);
        verify(
            waitForChild(window.contentItem, "urlPreviewDelegate.contextMenu"));
        var menu =
            findChild(window.contentItem, "urlPreviewDelegate.contextMenu");
        var deleteAction = findChild(menu, "delete_button");
        clickItem(deleteAction);
        compare(topSites.count, 0);
    }

    SignalSpy {
        id: positionChangedSpy
        signalName: "yChanged"
    }

    function test_wide_drag_bookmarks() {
        wide_init();
        wide_go_to_section(1);
        var foldersList = findChild(newTabView, "foldersList");
        var bookmarksList = findChild(newTabView, "bookmarksList");
        compare(bookmarksList.count, 5);

        // Test that when hovering normal bookmarks item the grip appears
        var bookmark = waitForChild(bookmarksList, "bookmarkItem_1");
        waitForRendering(bookmark);
        compare(bookmark.url, "%1/31/gallium".arg(bookmarksRoot));
        var grip = findChild(bookmark, "dragGrip");
        compare(grip.opacity, 0);
        mouseMove(grip, grip.width / 2, grip.height / 2);
        tryCompare(grip, "opacity", 1);

        // Test that an item bounces back when dragged within the list itself
        var position0 = Qt.point(bookmark.x, bookmark.y);
        positionChangedSpy.target = bookmark;
        var center = centerOf(grip);
        positionChangedSpy.clear();
        mouseDrag(grip, center.x, center.y, 0, 200);
        verify(positionChangedSpy.count > 0);
        compare(Qt.point(bookmark.x, bookmark.y), position0);

        // Test that an item bounces back when dragged to the same folder
        var folder = findChild(foldersList, "folderItem_0");
        var globalCenterOfFolder = folder.mapToGlobal(centerOf(folder));
        var globalCenterOfGrip = grip.mapToGlobal(center);
        positionChangedSpy.clear();
        mouseDrag(grip, center.x, center.y,
                  globalCenterOfFolder.x - globalCenterOfGrip.x,
                  globalCenterOfFolder.y - globalCenterOfGrip.y);
        verify(positionChangedSpy.count > 0);
        compare(Qt.point(bookmark.x, bookmark.y), position0);

        // Test that dragging an item to another folder removes it from this one
        // and adds it to the target folder
        folder = findChild(foldersList, "folderItem_1");
        globalCenterOfFolder = folder.mapToGlobal(centerOf(folder));
        positionChangedSpy.clear();
        mouseDrag(grip, center.x, center.y,
                  globalCenterOfFolder.x - globalCenterOfGrip.x,
                  globalCenterOfFolder.y - globalCenterOfGrip.y,
                  Qt.LeftButton, Qt.NoModifiers, 1);
        verify(positionChangedSpy.count > 0);
        tryCompare(bookmarksList, "count", 4);

        // Verify that the item has been added to the top of the target folder
        clickItem(folder);
        tryCompare(bookmarksList, "count", 2);
        bookmark = waitForChild(bookmarksList, "bookmarkItem_0");
        compare(bookmark.url, "%1/31/gallium".arg(bookmarksRoot));
    }
}
