/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "NewTabViewLifetime"

    property var newTabView: null

    function test_new_tab_view_destroyed_when_browsing() {
        open_new_tab();
        newTabView = findChild(window.contentItem, "newTabView");
        verify(newTabView != null);
        browse_to("test2.html", true);
        tryCompare(this, "newTabView", null);
    }

    function test_new_tab_view_destroyed_when_closing_tab() {
        open_new_tab();
        newTabView = findChild(window.contentItem, "newTabView");
        verify(newTabView != null);
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(this, "newTabView", null);
    }

    function test_new_tab_view_is_shared_between_tabs() {
        // Open one new tab
        open_new_tab();
        newTabView = findChild(window.contentItem, "newTabView");
        verify(newTabView != null);
        // Open a second new tab and verify that the new tab view is shared
        open_new_tab();
        verify(newTabView.visible);
        // Close the second new tab and verify that the new tab view is still there
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 2);
        verify(newTabView != null);
        // Close the first new tab and verify that the new tab view is destroyed
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(this, "newTabView", null);
    }

    function test_new_tab_view_not_destroyed_when_closing_last_open_tab() {
        if (window.wide) {
            // XXX: in wide mode, closing the last open tab closes the app,
            // which is not well handled by the test (it segfaults)
            skip("Only on narrow form factors");
        }
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(window.tabsModel, "currentIndex", 0);
        waitFor(function() { return (window.currentWebview != null); });
        newTabView = waitForChild(window.contentItem, "newTabView");
        verify(newTabView != null);
        // Verify that the new tab view is not destroyed and then re-created
        // when closing the last open tab if it was a blank one
        keyClick(Qt.Key_W, Qt.ControlModifier);
        tryCompare(window.tabsModel, "currentIndex", 0);
        waitFor(function() { return (window.currentWebview != null); });
        verify(newTabView != null);
        compare(waitForChild(window.contentItem, "newTabView"), newTabView);
    }

    function test_private_window_uses_private_tab_view() {
        var publicWindow = app.allWindows[0];
        open_drawer("newprivatewindow");
        tryCompare(app.allWindows, "length", 2);
        var privateWindow = app.allWindows[1];
        verify(privateWindow.incognito);
        tryCompare(privateWindow, "active", true);
        newTabView =
            waitForChild(privateWindow.contentItem, "newPrivateTabView");
        tryCompare(newTabView, "visible", true);

        // Clean up
        window = privateWindow;
        window.close();
        tryCompare(this, "window", null);
        window = publicWindow;
    }
}
