/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "Settings"

    property var settingsPage: null
    property var subPage: null
    property var dialog: null

    function open_settings() {
        open_drawer("settings");
        settingsPage = waitForChild(window.contentItem, "settingsPage");
        verify(settingsPage != null);
        waitForRendering(settingsPage);
        // Give the page some time to settle, because the visibility
        // of the "searchengine" list item is conditional.
        wait(100);
    }

    function test_open_close_settings_page() {
        open_settings();
        var pageHeader = findChild(settingsPage, "pageHeader");
        var backButton = waitForChild(pageHeader, "back_button");
        tryCompare(backButton, "visible", true);
        clickItem(backButton);
        tryCompare(this, "settingsPage", null);
    }

    function test_open_close_searchengine_page() {
        open_settings();
        var searchEngineEntry = findChild(settingsPage, "searchengine");
        tryCompare(searchEngineEntry, "visible", true);
        var oldEngine = searchEngineEntry.currentSearchEngineDisplayName;
        verify(oldEngine.length > 0);
        clickItem(searchEngineEntry);
        subPage = waitForChild(settingsPage, "searchEnginePage");
        var pageHeader = findChild(subPage, "pageHeader");
        var backButton = waitForChild(pageHeader, "back_button");
        tryCompare(backButton, "visible", true);
        clickItem(backButton);
        tryCompare(this, "subPage", null);
        compare(searchEngineEntry.currentSearchEngineDisplayName, oldEngine);
    }

    function test_change_searchengine() {
        open_settings();
        var searchEngineEntry = findChild(settingsPage, "searchengine");
        tryCompare(searchEngineEntry, "visible", true);
        var oldEngine = searchEngineEntry.currentSearchEngineDisplayName;
        verify(oldEngine.length > 0);
        clickItem(searchEngineEntry);
        subPage = waitForChild(settingsPage, "searchEnginePage");
        var listview = findChild(subPage, "searchEngineListView");
        // Needs to be updated when adding/removing search engine definitions
        tryCompare(listview, "count", 8);
        waitForRendering(listview);
        var current = null;
        var next = null;
        for (var i = 0; i < listview.count; ++i) {
            var delegate = findChild(listview, "searchEngineDelegate_%1".arg(i));
            if (delegate.checked) {
                verify(!current);
                current = delegate;
                compare(delegate.displayName, oldEngine);
            } else if (!next) {
                next = delegate;
            }
        }
        var newEngine = next.displayName;
        verify(newEngine != "");
        verify(newEngine != oldEngine);
        var checkbox = findChild(next, "checkbox");
        clickItem(checkbox);
        tryCompare(this, "subPage", null);
        tryCompare(searchEngineEntry, "currentSearchEngineDisplayName", newEngine);
    }

    function test_change_homepage() {
        open_settings();
        var homepageEntry = findChild(settingsPage, "homepage");
        var oldHomepage = homepageEntry.currentHomepage;
        verify(oldHomepage != "");

        // First test cancelling the edition
        clickItem(homepageEntry);
        dialog = waitForChild(window.contentItem, "homepageDialog");
        var textField = findChild(dialog, "homepageDialog.text");
        tryCompare(textField, "text", oldHomepage);
        var cancelButton = findChild(dialog, "homepageDialog.cancelButton");
        clickItem(cancelButton);
        tryCompare(this, "dialog", null);
        compare(homepageEntry.currentHomepage, oldHomepage);

        // Then test actually changing the homepage
        clickItem(homepageEntry);
        dialog = waitForChild(window.contentItem, "homepageDialog");
        textField = findChild(dialog, "homepageDialog.text");
        tryCompare(textField, "text", oldHomepage);
        tryCompare(textField, "activeFocus", true);
        var clearButton = findChild(textField, "clear_button");
        clickItem(clearButton);
        var newHomepage = "http://test/foo-bar-baz";
        typeString(newHomepage);
        var saveButton = findChild(dialog, "homepageDialog.saveButton");
        clickItem(saveButton);
        tryCompare(this, "dialog", null);
        tryCompare(homepageEntry, "currentHomepage", newHomepage);
    }

    function test_open_close_privacy_settings() {
        open_settings();
        var privacyEntry = findChild(settingsPage, "privacy");
        clickItem(privacyEntry);
        subPage = waitForChild(settingsPage, "privacySettings");
        var pageHeader = findChild(subPage, "pageHeader");
        var backButton = waitForChild(pageHeader, "back_button");
        tryCompare(backButton, "visible", true);
        clickItem(backButton);
        tryCompare(this, "subPage", null);
    }

    function test_clear_browsing_history() {
        open_settings();
        var privacyEntry = findChild(settingsPage, "privacy");
        clickItem(privacyEntry);
        subPage = waitForChild(settingsPage, "privacySettings");
        var clearHistoryEntry = findChild(subPage, "privacy.clearHistory");
        verify(clearHistoryEntry.enabled);

        // First test cancelling the action
        clickItem(clearHistoryEntry);
        dialog = waitForChild(window.contentItem, "privacyConfirmDialog");
        var cancelButton = findChild(dialog, "privacyConfirmDialog.cancelButton");
        clickItem(cancelButton);
        tryCompare(this, "dialog", null);
        verify(clearHistoryEntry.enabled);

        // Then confirm the action
        clickItem(clearHistoryEntry);
        dialog = waitForChild(window.contentItem, "privacyConfirmDialog");
        var confirmButton = findChild(dialog, "privacyConfirmDialog.confirmButton");
        clickItem(confirmButton);
        tryCompare(this, "dialog", null);
        tryCompare(clearHistoryEntry, "enabled", false);
    }

    function test_clear_cache() {
        open_settings();
        var privacyEntry = findChild(settingsPage, "privacy");
        clickItem(privacyEntry);
        subPage = waitForChild(settingsPage, "privacySettings");
        var clearCacheEntry = findChild(subPage, "privacy.clearCache");
        verify(clearCacheEntry.enabled);

        // First test cancelling the action
        clickItem(clearCacheEntry);
        dialog = waitForChild(window.contentItem, "privacyConfirmDialog");
        var cancelButton = findChild(dialog, "privacyConfirmDialog.cancelButton");
        clickItem(cancelButton);
        tryCompare(this, "dialog", null);
        verify(clearCacheEntry.enabled);

        // Then confirm the action
        clickItem(clearCacheEntry);
        dialog = waitForChild(window.contentItem, "privacyConfirmDialog");
        var confirmButton = findChild(dialog, "privacyConfirmDialog.confirmButton");
        clickItem(confirmButton);
        tryCompare(this, "dialog", null);
        tryCompare(clearCacheEntry, "enabled", true);
    }

    function test_reset_browser_settings() {
        open_settings();
        var resetSettingsEntry = findChild(settingsPage, "reset");
        clickItem(resetSettingsEntry);

        var searchEngineEntry = findChild(settingsPage, "searchengine");
        tryCompare(searchEngineEntry, "currentSearchEngineDisplayName", "Google");

        var homepageEntry = findChild(settingsPage, "homepage");
        tryCompare(homepageEntry, "currentHomepage", "http://start.ubuntu.com");

        var restoreSessionEntry = findChild(settingsPage, "restoreSession");
        tryCompare(restoreSessionEntry, "checked", true);
    }
}
