/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "Suggestions"

    historyDbFile: Qt.resolvedUrl("suggestions-history.sqlite")
    bookmarksDbFile: Qt.resolvedUrl("suggestions-bookmarks.sqlite")
    configFile: Qt.resolvedUrl("suggestions-config.conf")
    searchEngineDescriptionFile: "searchengines/suggestions-test.xml"
    searchEngineDescription: "<OpenSearchDescription><Url type=\"application/x-suggestions+json\" template=\"%1suggest.py?q={searchTerms}\"/><Url type=\"text/html\" template=\"http://aserver.somewhere/search?q={searchTerms}\"/></OpenSearchDescription>".arg(baseUrl)

    function get_suggestions() {
        return findChild(window.contentItem, "suggestionsList");
    }

    function assert_suggestions_eventually_shown() {
        tryCompare(get_suggestions(), "opacity", 1.0);
    }

    function assert_suggestions_eventually_hidden() {
        tryCompare(get_suggestions(), "opacity", 0.0);
    }

    function get_suggestion(index) {
        return findChild(get_suggestions(), "suggestionDelegate_%1".arg(index));
    }

    function unhighlight(highlighted) {
        // Remove HTML markup from a highlighted string
        return highlighted.replace(/<(?:.|\n)*?>/gm, "");
    }

    function test_show_list_of_suggestions() {
        var suggestions = get_suggestions();
        assert_suggestions_eventually_hidden();
        focus_address_bar(false);
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 1);
        clear_address_bar();
        assert_suggestions_eventually_hidden();
    }

    function test_list_of_suggestions_case_insensitive() {
        var suggestions = get_suggestions();
        focus_address_bar(true);
        typeString("SpEciAl");
        tryCompare(suggestions, "count", 1);
    }

    function test_list_of_suggestions_limits_data() {
        return [
            {strings: ["ubuntu", "bleh", "iana"]},
            {strings: ["element", "bleh", "linux"]},
        ];
    }

    function test_list_of_suggestions_limits(data) {
        var suggestions = get_suggestions();
        focus_address_bar(true);
        typeString(data.strings[0]);
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 2);
        typeString(data.strings[1]);
        tryCompare(suggestions, "count", 0);
        clear_address_bar();
        typeString(data.strings[2]);
        tryCompare(suggestions, "count", 1);
    }

    function test_list_of_suggestions_search_limits() {
        var suggestions = get_suggestions();
        focus_address_bar(true);
        typeString("foo");
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 4);
        typeString("bleh");
        tryCompare(suggestions, "count", 0);
    }

    function test_list_of_suggestions_order() {
        var suggestions = get_suggestions();
        focus_address_bar(true);
        typeString("QML");
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 5);
        compare(get_suggestion(0).icon, "history");
        compare(get_suggestion(1).icon, "non-starred");
        compare(get_suggestion(2).icon, "search");
    }

    function test_clear_address_bar_dismisses_suggestions() {
        focus_address_bar(true);
        typeString("ubuntu");
        assert_suggestions_eventually_shown();
        clear_address_bar();
        assert_suggestions_eventually_hidden();
    }

    function test_addressbar_loosing_focus_dismisses_suggestions() {
        var suggestions = get_suggestions();
        focus_address_bar(false);
        assert_suggestions_eventually_shown();
        // Click somewhere in the webview but outside the suggestions list
        var webview = window.currentWebview;
        mouseClick(webview, webview.width / 2, (suggestions.y + suggestions.height + webview.height) / 2);
        assert_suggestions_eventually_hidden();
    }

    function test_suggestions_hidden_while_drawer_open() {
        focus_address_bar(false);
        assert_suggestions_eventually_shown();
        var drawer = open_drawer(null);
        assert_suggestions_eventually_hidden();
        var chrome = findChild(window.contentItem, "chrome");
        var drawerButton = findChild(chrome, "drawerButton");
        clickItem(drawerButton);
        assert_suggestions_eventually_shown();
    }

    function test_select_suggestion() {
        var suggestions = get_suggestions();
        focus_address_bar(true);
        typeString("linux");
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 1);
        clickItem(get_suggestion(0));
        assert_suggestions_eventually_hidden();
        tryCompare(window.currentWebview, "url", "http://test/wiki/Linux");
    }

    function test_select_search_suggestion() {
        var suggestions = get_suggestions();
        focus_address_bar(true);
        typeString("high");
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 1);
        clickItem(get_suggestion(0));
        assert_suggestions_eventually_hidden();
        tryCompare(window.currentWebview, "url", "http://aserver.somewhere/search?q=highlight");
    }

    function test_special_characters() {
        var suggestions = get_suggestions();
        focus_address_bar(true);
        typeString("(phil");
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 1);
        var suggestion = get_suggestion(0);
        compare(unhighlight(suggestion.title),
                "Ubuntu (philosophy) - Wikipedia, the free encyclopedia");
        compare(unhighlight(suggestion.subtitle),
                "http://en.wikipedia.org/wiki/Ubuntu_(philosophy)");
    }

    function test_search_suggestions() {
        var suggestions = get_suggestions();
        focus_address_bar(true);
        typeString("high");
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 1);
        var suggestion = get_suggestion(0);
        compare(unhighlight(suggestion.title), "highlight");
        compare(suggestion.subtitle, "");
    }

    function test_keyboard_navigation() {
        var suggestions = get_suggestions();
        var addressbar = focus_address_bar(true);
        typeString("element");
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 2);
        var suggestion1 = get_suggestion(0);
        var suggestion2 = get_suggestion(1);
        verify(!suggestion1.selected);
        verify(!suggestion2.selected);

        keyClick(Qt.Key_Down);
        tryCompare(addressbar, "activeFocus", false);
        tryCompare(suggestions, "activeFocus", true);
        verify(suggestion1.selected);
        verify(!suggestion2.selected);

        keyClick(Qt.Key_Down);
        verify(!suggestion1.selected);
        verify(suggestion2.selected);

        // Verify that selection does not wrap around
        keyClick(Qt.Key_Down);
        verify(!suggestion1.selected);
        verify(suggestion2.selected);

        keyClick(Qt.Key_Up);
        verify(suggestion1.selected);
        verify(!suggestion2.selected);

        keyClick(Qt.Key_Up);
        tryCompare(addressbar, "activeFocus", true);
        tryCompare(suggestions, "activeFocus", false);
        verify(!suggestion1.selected);
        verify(!suggestion2.selected);
    }

    function test_suggestions_escape() {
        var suggestions = get_suggestions();
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        var text = addressbar.text;
        focus_address_bar(false);
        typeString("element");
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 2);
        keyClick(Qt.Key_Down);
        tryCompare(suggestions, "activeFocus", true);
        tryCompare(addressbar, "text", "element");
        keyClick(Qt.Key_Escape);
        assert_suggestions_eventually_hidden();
        tryCompare(addressbar, "text", text);
    }

    function test_suggestions_escape_on_addressbar() {
        var suggestions = get_suggestions();
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressBarTextField");
        var text = addressbar.text;
        focus_address_bar(false);
        typeString("element");
        assert_suggestions_eventually_shown();
        tryCompare(suggestions, "count", 2);
        keyClick(Qt.Key_Down);
        tryCompare(suggestions, "activeFocus", true);
        keyClick(Qt.Key_Up);
        tryCompare(addressbar, "activeFocus", true);
        keyClick(Qt.Key_Escape);
        assert_suggestions_eventually_hidden();
        tryCompare(addressbar, "text", text);
    }
}
