/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Qt
#include <QtCore/QDebug>
#include <QtCore/QDir>
#include <QtCore/QFile>
#include <QtCore/QFileInfo>
#include <QtCore/QProcess>
#include <QtCore/QRegExp>
#include <QtCore/QSize>
#include <QtCore/QStandardPaths>
#include <QtCore/QString>
#include <QtCore/QTemporaryDir>
#include <QtCore/QtGlobal>
#include <QtCore/QUrl>
#include <QtGui/QGuiApplication>
#include <QtQml/QtQml>
#include <QtQuickTest/QtQuickTest>
#include <QtTest/QTest>

// webbrowser-common
#include "favicon-fetcher.h"
#include "meminfo.h"
#include "mime-database.h"
#include "qquickshortcut_p.h"
#include "session-storage.h"

// webbrowser-app
#include "bookmarks-model.h"
#include "bookmarks-folder-model.h"
#include "bookmarks-folderlist-model.h"
#include "cache-deleter.h"
#include "downloads-model.h"
#include "favicon-fetcher.h"
#include "file-operations.h"
#include "history-domain-model.h"
#include "history-domainlist-model.h"
#include "history-model.h"
#include "history-lastvisitdatelist-model.h"
#include "limit-proxy-model.h"
#include "reparenter.h"
#include "searchengine.h"
#include "tabs-model.h"
#include "text-search-filter-model.h"

// local
#include "testconfig.h"

class TestContext : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QSize initialWindowSize READ initialWindowSize CONSTANT)
    Q_PROPERTY(int httpServerPort READ httpServerPort CONSTANT)
    Q_PROPERTY(QString searchEnginesSearchPath READ searchEnginesSearchPath CONSTANT)
    Q_PROPERTY(QString platformName READ platformName CONSTANT)

public:
    int httpServerPort() const
    {
        return testHttpServerPort;
    }

    QString searchEnginesSearchPath() const
    {
        QDir dir(QStandardPaths::writableLocation(QStandardPaths::DataLocation));
        return dir.absoluteFilePath("searchengines");
    }

    QSize initialWindowSize() const
    {
        return testWindowSize;
    }

    QString platformName() const
    {
        return QGuiApplication::platformName();
    }

    Q_INVOKABLE bool copyDataFile(const QString& filepath, const QString& filename) const
    {
        return copyFile(filepath, QStandardPaths::DataLocation, filename);
    }

    Q_INVOKABLE bool writeDataFile(const QString& contents, const QString& filename) const
    {
        QDir dir(QStandardPaths::writableLocation(QStandardPaths::DataLocation));
        QFileInfo info(dir.absoluteFilePath(filename));
        dir = info.absoluteDir();
        if (!dir.exists()) {
            QDir::root().mkpath(dir.absolutePath());
        }
        QFile file(info.absoluteFilePath());
        if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
            return false;
        }
        if (file.write(contents.toUtf8()) == -1) {
            return false;
        }
        file.close();
        return true;
    }

    Q_INVOKABLE bool removeDataFile(const QString& filename) const
    {
        return removeFile(QStandardPaths::DataLocation, filename);
    }

    Q_INVOKABLE bool copyConfigFile(const QString& filepath, const QString& filename) const
    {
        return copyFile(filepath, QStandardPaths::ConfigLocation, filename);
    }

    Q_INVOKABLE bool removeConfigFile(const QString& filename) const
    {
        return removeFile(QStandardPaths::ConfigLocation, filename);
    }

    Q_INVOKABLE bool copyCacheFile(const QString& filepath, const QString& filename) const
    {
        return copyFile(filepath, QStandardPaths::CacheLocation, filename);
    }

    Q_INVOKABLE bool checkCacheFileExists(const QString& filename) const
    {
        return checkFileExists(QStandardPaths::CacheLocation, filename);
    }

    Q_INVOKABLE bool runContentHubPeerHookWrapper() const
    {
        QProcess wrapper;
        QString exec("/usr/lib/%1/content-hub/content-hub-peer-hook-wrapper");
        wrapper.start(exec.arg(LibraryArchitecture()));
        return wrapper.waitForFinished();
    }

    static int testHttpServerPort;
    static QSize testWindowSize;

private:
    bool copyFile(const QString& filepath,
                  QStandardPaths::StandardLocation location,
                  const QString& filename) const
    {
        QFile file(QUrl(filepath).toLocalFile());
        QDir dir(QStandardPaths::writableLocation(location));
        QFileInfo info(dir.absoluteFilePath(filename));
        dir = info.absoluteDir();
        if (!dir.exists()) {
            QDir::root().mkpath(dir.absolutePath());
        }
        return file.copy(dir.absoluteFilePath(info.fileName()));
    }

    bool removeFile(QStandardPaths::StandardLocation location,
                    const QString& filename) const
    {
        QDir dir(QStandardPaths::writableLocation(location));
        // Work around racy QDir::delete():
        while (QFileInfo(dir.absoluteFilePath(filename)).exists()) {
            dir.remove(filename);
            QTest::qWait(1);
        }
        return true;
    }

    bool checkFileExists(QStandardPaths::StandardLocation location,
                         const QString& filename) const
    {
        QDir dir(QStandardPaths::writableLocation(location));
        return QFileInfo(dir.absoluteFilePath(filename)).exists();
    }
};

int TestContext::testHttpServerPort = 0;
QSize TestContext::testWindowSize = QSize();

#define MAKE_SINGLETON_FACTORY(type) \
    static QObject* type##_singleton_factory(QQmlEngine* engine, QJSEngine* scriptEngine) { \
        Q_UNUSED(engine); \
        Q_UNUSED(scriptEngine); \
        return new type(); \
    }

MAKE_SINGLETON_FACTORY(BookmarksModel)
MAKE_SINGLETON_FACTORY(CacheDeleter)
MAKE_SINGLETON_FACTORY(DownloadsModel)
MAKE_SINGLETON_FACTORY(FileOperations)
MAKE_SINGLETON_FACTORY(HistoryModel)
MAKE_SINGLETON_FACTORY(MemInfo)
MAKE_SINGLETON_FACTORY(MimeDatabase)
MAKE_SINGLETON_FACTORY(Reparenter)


static QObject* TestContext_singleton_factory(QQmlEngine* engine, QJSEngine* scriptEngine) {
    Q_UNUSED(engine)
    Q_UNUSED(scriptEngine)
    TestContext* testContext = new TestContext;
    return testContext;
}

#define REGISTER_TYPE(uri, type) \
    qmlRegisterType<type>(uri, 0, 1, #type);
#define REGISTER_SINGLETON_TYPE(uri, type) \
    qmlRegisterSingletonType<type>(uri, 0, 1, #type, type##_singleton_factory);

int main(int argc, char** argv)
{
    // Intercept special test parameters
    QList<char*> filteredArgv;
    QString initialWindowSizeToken(QStringLiteral("--initialWindowSize="));
    for (int i = 0; i < argc; ++i) {
        QString arg(argv[i]);
        if (arg.startsWith(initialWindowSizeToken)) {
            bool valid = false;
            QStringList sizeArgs = arg.split(initialWindowSizeToken);
            if (sizeArgs.size() == 2) {
                QString sizeSpec = sizeArgs.at(1);
                sizeArgs = sizeSpec.split(QStringLiteral("x"));
                if (sizeArgs.size() == 2) {
                    bool ok;
                    int width = sizeArgs.at(0).toInt(&ok);
                    if (ok) {
                        int height = sizeArgs.at(1).toInt(&ok);
                        if (ok) {
                            TestContext::testWindowSize = QSize(width, height);
                            valid = true;
                        }
                    }
                }
            }
            if (!valid) {
                qWarning() << "Initial window size not correctly specified.";
            }
        } else {
            filteredArgv.append(argv[i]);
        }
    }
    int testArgc = filteredArgv.size();
    QScopedArrayPointer<char*> testArgv(new char*[testArgc + 1]);
    for (int i = 0; i < testArgc; ++i) {
        testArgv[i] = filteredArgv.at(i);
    }
    testArgv[testArgc] = 0;

    qmlRegisterSingletonType<TestContext>("webbrowser.qmltests", 0, 1, "TestContext",
                                          TestContext_singleton_factory);

    const char* commonUri = "webbrowsercommon.private";
    REGISTER_TYPE(commonUri, FaviconFetcher)
    REGISTER_SINGLETON_TYPE(commonUri, MemInfo)
    REGISTER_SINGLETON_TYPE(commonUri, MimeDatabase)
    REGISTER_TYPE(commonUri, SessionStorage)
    qmlRegisterType<QQuickShortcut>(commonUri, 0, 1, "Shortcut");

    const char* browserUri = "webbrowserapp.private";
    REGISTER_SINGLETON_TYPE(browserUri, BookmarksModel);
    REGISTER_TYPE(browserUri, BookmarksFolderListModel);
    REGISTER_SINGLETON_TYPE(browserUri, CacheDeleter);
    REGISTER_SINGLETON_TYPE(browserUri, DownloadsModel);
    REGISTER_SINGLETON_TYPE(browserUri, FileOperations);
    REGISTER_SINGLETON_TYPE(browserUri, HistoryModel);
    REGISTER_TYPE(browserUri, HistoryDomainModel);
    REGISTER_TYPE(browserUri, HistoryDomainListModel);
    REGISTER_TYPE(browserUri, HistoryLastVisitDateListModel);
    REGISTER_TYPE(browserUri, LimitProxyModel);
    REGISTER_SINGLETON_TYPE(browserUri, Reparenter);
    REGISTER_TYPE(browserUri, SearchEngine);
    REGISTER_TYPE(browserUri, TabsModel);
    REGISTER_TYPE(browserUri, TextSearchFilterModel);

    qputenv("APP_ID", QStringLiteral("webbrowser-app").toUtf8());

    QStandardPaths::setTestModeEnabled(true);
    // QStandardPaths::setTestModeEnabled() doesn't affect
    // QStandardPaths::DownloadLocation, so we must override $HOME to
    // ensure the test won't write data to the user's home directory.
    QTemporaryDir homeDir;
    qputenv("HOME", homeDir.path().toUtf8());

    // Ensure the existence of fake devices for media streams
    qputenv("OXIDE_TESTING_MODE", "1");

    // Start test HTTP server
    QProcess httpServer;
    httpServer.setReadChannel(QProcess::StandardOutput);
    QStringList args;
    args << QStringLiteral("-B")
         << TestDirectory().absoluteFilePath("testserver.py");
    httpServer.start(QStringLiteral("python"), args);
    httpServer.waitForStarted();
    httpServer.waitForReadyRead();
    QRegExp portParser("^PORT:(\\d+)$");
    if (portParser.exactMatch(httpServer.readAllStandardOutput())) {
        TestContext::testHttpServerPort = portParser.cap(1).toInt();
    } else {
        qWarning() << "Failed to run test HTTP server";
        return 1;
    }

    QCoreApplication::setOrganizationDomain(QStringLiteral("webbrowser-app-qmltests"));
    int result = quick_test_main(testArgc, testArgv.data(), "webbrowser-app-qmltests", nullptr);
    httpServer.terminate();
    httpServer.waitForFinished();
    return result;
}

#include "webbrowser-app-qmltests-launcher.moc"
